#include "WiFiNetwork.h"


// Constructor of Class WiFiNetwork   (function name = class name)
WiFiNetwork::WiFiNetwork()
{
	
}


// Put ESP32 in both modes in parallel: Soft Access Point and station in router network (LAN)
int WiFiNetwork::SetBothModesNetworkStationAndAccessPoint()
{
  WiFi.mode(WIFI_AP_STA);
  
  return 1;
}


// Connect to router network and return 1 (success) or -1 (no success)
int WiFiNetwork::RouterNetworkConnect(char* txtSSID, char* txtPassword)
{
  int success = 1;
  
  // connect to WiFi network
  // see https://www.arduino.cc/en/Reference/WiFiBegin
  
  WiFi.begin(txtSSID, txtPassword);
  
  // we wait until connection is established
  // or 10 seconds are gone
  
  int WiFiConnectTimeOut = 0;
  while ((WiFi.status() != WL_CONNECTED) && (WiFiConnectTimeOut < 10))
  {
     // try it 10 times a second
    delay(1000);
    WiFiConnectTimeOut++;
	
	//Serial.println(WiFi.status());
  }

  // not connected
  if (WiFi.status() != WL_CONNECTED)
  {
    success = -1;
  }

  return success;
}



// Disconnect from router network and return 1 (success) or -1 (no success)
int WiFiNetwork::RouterNetworkDisconnect()
{
  int success = -1;
  
  // try to disconnect from network if connected
  if (WiFi.status() == WL_CONNECTED)
  {
	WiFi.disconnect();
  }
  
  int WiFiConnectTimeOut = 0;
  while ((WiFi.status() == WL_CONNECTED) && (WiFiConnectTimeOut < 10))
  {
    // try it 10 times a second
    delay(1000);
    WiFiConnectTimeOut++;
  }

  // not connected
  if (WiFi.status() != WL_CONNECTED)
  {
    success = 1;
  }
  
  return success;
}


// Initialize Soft Access Point with ESP32
// ESP32 establishes its own WiFi network, one can choose the SSID
int WiFiNetwork::AccessPointStart(char* AccessPointNetworkSSID)
{ 
  WiFi.softAP(AccessPointNetworkSSID);

  return 1;
}


// Get IP-Address of ESP32 in Router network (LAN), in String-format
String WiFiNetwork::GetOwnIPAddressInRouterNetwork()
{
  return WiFi.localIP().toString();
}



// Get IP-Address of ESP32 in own network, in String-format
String WiFiNetwork::GetIPAddressInOwnNetwork()
{
  return WiFi.softAPIP().toString();
};

